package ru.dbotthepony.mc.otm.datagen.blocks

import net.minecraft.resources.ResourceLocation
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.block.state.properties.Property
import net.minecraftforge.client.model.generators.BlockStateProvider
import net.minecraftforge.client.model.generators.ConfiguredModel
import net.minecraftforge.forge.event.lifecycle.GatherDataEvent
import ru.dbotthepony.mc.otm.block.BlockMattery
import ru.dbotthepony.mc.otm.block.BlockMatteryRotatable
import ru.dbotthepony.mc.otm.block.entity.worker.WorkerState
import ru.dbotthepony.mc.otm.datagen.DataGen
import ru.dbotthepony.mc.otm.datagen.getValueNullable
import ru.dbotthepony.mc.otm.datagen.toYRotBlockstate

typealias AdvancedBlockStateFunction = (BlockState, ConfiguredModel.Builder<*>, String) -> String?
//typealias AdvancedBlockStateBoolFunction = (BlockState, ConfiguredModel.Builder<*>) -> String

private data class AdvancedBlockStateEntry(val block: BlockMattery, val func: AdvancedBlockStateFunction)
//data class BooleanPropertyMapping(val property: Property<Boolean>, val func: AdvancedBlockStateBoolFunction)
//private data class AdvancedBlockStateBooleanMapped(val block: BlockMattery, val mapping: Array<out BooleanPropertyMapping>)

private fun initialTransform(it: BlockState, modelPath: String, builder: ConfiguredModel.Builder<*>): String {
	var modelPath = modelPath

	it.getValueNullable(BlockMatteryRotatable.FACING)?.let {
		builder.rotationY(it.toYRotBlockstate())
	}

	it.getValueNullable(WorkerState.WORKER_STATE)?.let {
		modelPath += "_" + it.name.lowercase()
	}

	it.getValueNullable(WorkerState.SEMI_WORKER_STATE)?.let {
		modelPath += "_" + it.name.lowercase()
	}

	return modelPath
}

class AdvancedBlockStateProvider(event: GatherDataEvent) : BlockStateProvider(event.generator, DataGen.MOD_ID, event.existingFileHelper) {
	private val blocks = ArrayList<AdvancedBlockStateEntry>()
	//private val boolMapped = ArrayList<AdvancedBlockStateBooleanMapped>()

	fun block(block: BlockMattery) = blocks.add(AdvancedBlockStateEntry(block) { _, _, _ -> null })
	fun block(block: BlockMattery, func: AdvancedBlockStateFunction) = blocks.add(AdvancedBlockStateEntry(block, func))

	//fun multipartBoolBlock(block: BlockMattery, vararg props: BooleanPropertyMapping) {
	//	boolMapped.add(AdvancedBlockStateBooleanMapped(block, props))
	//}

	override fun registerStatesAndModels() {
		for ((block, func) in blocks) {
			getVariantBuilder(block).forAllStates {
				val builder = ConfiguredModel.builder()
				var modelPath = initialTransform(it, "block/${block.registryName!!.path}", builder)
				modelPath = func(it, builder, modelPath) ?: modelPath

				builder.modelFile(models().getExistingFile(ResourceLocation(DataGen.MOD_ID, modelPath)))

				return@forAllStates arrayOf(builder.buildLast())
			}
		}
	}
}
